import datetime

import pytest

from program.tests.factories import EpisodeFactory, TimeslotFactory

pytestmark = pytest.mark.django_db


def url(timeslot=None) -> str:
    if timeslot:
        return f"/api/v1/timeslots/{timeslot.id}/"
    else:
        return "/api/v1/timeslots/"


def test_delete_timeslot_as_admin(admin_api_client, once_timeslot):
    response = admin_api_client.delete(url(timeslot=once_timeslot))

    assert response.status_code == 204


def test_delete_forbidden_as_common_user(common_api_client1, once_timeslot):
    response = common_api_client1.delete(url(timeslot=once_timeslot))

    assert response.status_code == 403


def test_delete_forbidden_as_unauthenticated_user(api_client, once_timeslot):
    response = api_client.delete(url(timeslot=once_timeslot))

    assert response.status_code == 403


def test_retrieve_timeslot(api_client, once_timeslot):
    response = api_client.get(url(timeslot=once_timeslot))

    assert response.status_code == 200


def test_update_memo_as_admin(admin_api_client, once_timeslot):
    update = {"memo": "MEMO"}

    response = admin_api_client.patch(url(timeslot=once_timeslot), data=update)

    assert response.status_code == 200


def test_update_playlist_id_as_admin(admin_api_client, once_timeslot, default_media, show):
    update = {"playlist_id": default_media.id}

    response = admin_api_client.patch(url(timeslot=once_timeslot), data=update)

    assert response.status_code == 200


def test_update_repetition_of_as_admin(admin_api_client, once_timeslot):
    update = {"repetition_of": 1}
    response = admin_api_client.patch(url(timeslot=once_timeslot), data=update)

    assert response.status_code == 200


def test_update_as_admin(admin_api_client, once_timeslot, default_media, show):
    episode = EpisodeFactory(show=once_timeslot.schedule.show)
    response = admin_api_client.put(url(timeslot=once_timeslot), data={"episodeId": episode.id})
    assert response.status_code == 200


def test_update_forbidden_as_not_owner(common_api_client2, owned_show_once_timeslot):
    episode = EpisodeFactory(show=owned_show_once_timeslot.schedule.show)
    response = common_api_client2.put(
        url(owned_show_once_timeslot), data={"episodeId": episode.id}
    )
    assert response.status_code == 403


def test_update_timeslot_forbidden_as_unauthenticated(api_client, once_timeslot):
    episode = EpisodeFactory(show=once_timeslot.schedule.show)
    response = api_client.put(url(timeslot=once_timeslot), data={"episodeId": episode.id})
    assert response.status_code == 403


def test_timeslot_repetition_of_is_filled_with_first_broadcast_of_episode(
    admin_api_client, owned_show_once_timeslot
):
    # timeslots are named broadcast in this test to make it easier to grasp their
    # meaning in the context of repetitions.
    schedule = owned_show_once_timeslot.schedule
    episode = EpisodeFactory(show=schedule.show)

    # the first broadcast should not have a repetition
    first_broadcast = owned_show_once_timeslot
    first_broadcast.episode = episode
    first_broadcast.save()
    response = admin_api_client.get(url(first_broadcast))
    assert response.status_code == 200, response.content.decode()
    assert response.data["repetition_of_id"] is None

    # the second broadcast should be a repetition of the first broadcast
    second_broadcast_start = first_broadcast.start + datetime.timedelta(weeks=1)
    second_broadcast = TimeslotFactory.create(
        schedule=schedule,
        start=second_broadcast_start,
        end=second_broadcast_start + datetime.timedelta(hours=1),
    )
    response = admin_api_client.patch(url(second_broadcast), data={"episodeId": episode.id})
    assert response.status_code == 200, response.content.decode()
    assert response.data["repetition_of_id"] == first_broadcast.id

    # the third broadcast should still be a repetition of the first broadcast
    third_broadcast_start = second_broadcast.start + datetime.timedelta(weeks=1)
    third_broadcast = TimeslotFactory.create(
        schedule=schedule,
        start=third_broadcast_start,
        end=third_broadcast_start + datetime.timedelta(hours=1),
    )
    response = admin_api_client.patch(url(third_broadcast), data={"episodeId": episode.id})
    assert response.status_code == 200, response.content.decode()
    assert response.data["repetition_of_id"] == first_broadcast.id

    # Now check whether the second broadcast of the episode becomes the first broadcast
    # if the episode assignment of first_broadcast is removed.
    first_broadcast.episode = None
    first_broadcast.save()
    response = admin_api_client.get(url(second_broadcast))
    assert response.status_code == 200, response.content.decode()
    assert response.data["repetition_of_id"] is None
    response = admin_api_client.get(url(third_broadcast), data={"episodeId": episode.id})
    assert response.status_code == 200, response.content.decode()
    assert response.data["repetition_of_id"] == second_broadcast.id
